@@ -1,4 +1,144 @@
-##########################################################################################################
+"""
+Tools to specify functions through trees and forests.
+
+Whaaa?!?
+
+Well, you see, often -- especially when writing transformers -- you have a series of
+if/then conditions nested into eachother, in code, where it gets ugly and un-reusable.
+
+This module explores ways to objectivy this: That is, to give us the means to create
+such nested conditions in a way that we can define the parts as reusable operable
+components.
+
+Think of the relationship between the for loop (code) and the iterator (object), along
+with iterator tools (itertools).
+This is what we're trying to explore, but for if/then conditions.
+
+I said explore. Some more work is needed here to make it easily usable.
+
+Let's look at an example involving the three main actors of our play.
+Each of these are ``Iterable`` and ``Callable`` (``Generator`` to be precise).
+
+- ``CondNode``: implements the if/then (no else) logic
+- ``FinalNode``: Final -- yields (both with call and iter) it's single `.val` attribute.
+- ``RoutingForest``: An Iterable of ``CondNode``
+
+>>> import inspect
+>>>
+>>> def could_be_int(obj):
+...     if isinstance(obj, int):
+...         b = True
+...     else:
+...         try:
+...             int(obj)
+...             b = True
+...         except ValueError:
+...             b = False
+...     if b:
+...         print(f'{inspect.currentframe().f_code.co_name}')
+...     return b
+...
+>>> def could_be_float(obj):
+...     if isinstance(obj, float):
+...         b = True
+...     else:
+...         try:
+...             float(obj)
+...             b = True
+...         except ValueError:
+...             b = False
+...     if b:
+...         print(f'{inspect.currentframe().f_code.co_name}')
+...     return b
+...
+>>> print(
+...     could_be_int(30),
+...     could_be_int(30.3),
+...     could_be_int('30.2'),
+...     could_be_int('nope'),
+... )
+could_be_int
+could_be_int
+True True False False
+>>> print(
+...     could_be_float(30),
+...     could_be_float(30.3),
+...     could_be_float('30.2'),
+...     could_be_float('nope'),
+... )
+could_be_float
+could_be_float
+could_be_float
+True True True False
+>>> assert could_be_int('30.2') is False
+>>> assert could_be_float('30.2') is True
+could_be_float
+>>>
+>>> st = RoutingForest(
+...     [
+...         CondNode(
+...             cond=could_be_int,
+...             then=RoutingForest(
+...                 [
+...                     CondNode(
+...                         cond=lambda x: int(x) >= 10,
+...                         then=FinalNode('More than a digit'),
+...                     ),
+...                     CondNode(
+...                         cond=lambda x: (int(x) % 2) == 1,
+...                         then=FinalNode("That's odd!"),
+...                     ),
+...                 ]
+...             ),
+...         ),
+...         CondNode(cond=could_be_float, then=FinalNode('could be seen as a float')),
+...     ]
+... )
+>>> assert list(st('nothing I can do with that')) == []
+>>> assert list(st(8)) == ['could be seen as a float']
+could_be_int
+could_be_float
+>>> assert list(st(9)) == ["That's odd!", 'could be seen as a float']
+could_be_int
+could_be_float
+>>> assert list(st(10)) == ['More than a digit', 'could be seen as a float']
+could_be_int
+could_be_float
+>>> assert list(st(11)) == [
+...     'More than a digit',
+...     "That's odd!",
+...     'could be seen as a float',
+... ]
+could_be_int
+could_be_float
+>>>
+>>> print(
+...     '### RoutingForest ########################################################################################'
+... )
+### RoutingForest ########################################################################################
+>>> rf = RoutingForest(
+...     [
+...         SwitchCaseNode(
+...             switch=lambda x: x % 5,
+...             cases={0: FinalNode('zero_mod_5'), 1: FinalNode('one_mod_5')},
+...             default=FinalNode('default_mod_5'),
+...         ),
+...         SwitchCaseNode(
+...             switch=lambda x: x % 2,
+...             cases={0: FinalNode('even'), 1: FinalNode('odd')},
+...             default=FinalNode('that is not an int'),
+...         ),
+...     ]
+... )
+>>>
+>>> assert list(rf(5)) == ['zero_mod_5', 'odd']
+>>> assert list(rf(6)) == ['one_mod_5', 'even']
+>>> assert list(rf(7)) == ['default_mod_5', 'odd']
+>>> assert list(rf(8)) == ['default_mod_5', 'even']
+>>> assert list(rf(10)) == ['zero_mod_5', 'even']
+>>>
+
+"""
 from itertools import chain
 from dataclasses import dataclass
 from typing import Any, Iterable, Callable, Mapping, Tuple
@@ -13,7 +153,8 @@ class RoutingNode:
 
 @dataclass
 class FinalNode(RoutingNode):
-    """A RoutingNode that is final. It yields (both with call and iter) it's single `.val` attribute."""
+    """A RoutingNode that is final.
+    It yields (both with call and iter) it's single `.val` attribute."""
 
     val: Any
 
